@@ -4,7 +4,7 @@
  * Copyright (C) 2002, 2003  Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -35,6 +35,12 @@
 
 DBUS_BEGIN_DECLS
 
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
+#define DBUS_EMBEDDED_TESTS_EXPORT DBUS_PRIVATE_EXPORT
+#else
+#define DBUS_EMBEDDED_TESTS_EXPORT /* nothing */
+#endif
+
 DBUS_PRIVATE_EXPORT
 void _dbus_warn               (const char *format,
                                ...) _DBUS_GNUC_PRINTF (1, 2);
@@ -113,6 +119,7 @@ DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_get_verbose (void);
 DBUS_PRIVATE_EXPORT
 void _dbus_set_verbose (dbus_bool_t state);
+void _dbus_verbose_raw (const char *s);
 
 #  define _dbus_verbose_reset _dbus_verbose_reset_real
 #  define _dbus_is_verbose _dbus_is_verbose_real
@@ -405,9 +412,6 @@ void        _dbus_unlock (DBusGlobalLock lock);
 #define _DBUS_LOCK(name)                _dbus_lock   (_DBUS_LOCK_##name)
 #define _DBUS_UNLOCK(name)              _dbus_unlock (_DBUS_LOCK_##name)
 
-DBUS_PRIVATE_EXPORT
-dbus_bool_t _dbus_threads_init_debug (void);
-
 DBUS_PRIVATE_EXPORT
 dbus_bool_t   _dbus_address_append_escaped (DBusString       *escaped,
                                             const DBusString *unescaped);
@@ -456,6 +460,9 @@ dbus_bool_t _dbus_get_local_machine_uuid_encoded (DBusString *uuid_str,
   typedef struct { char _assertion[(expr) ? 1 : -1]; } \
   _DBUS_PASTE (_DBUS_STATIC_ASSERT_, __LINE__) _DBUS_GNUC_UNUSED
 
+#define _DBUS_STRINGIFY(x) #x
+#define _DBUS_FILE_LINE __FILE__ ":" _DBUS_STRINGIFY(__LINE__)
+
 DBUS_END_DECLS
 
 #endif /* DBUS_INTERNALS_H */
