@@ -99,6 +99,7 @@
  */
 
 #ifdef DBUS_ENABLE_EMBEDDED_TESTS
+/* Test-only, does not need to be thread-safe */
 static dbus_bool_t debug_initialized = FALSE;
 static int fail_nth = -1;
 static size_t fail_size = 0;
@@ -260,7 +261,7 @@ _dbus_decrement_fail_alloc_counter (void)
 
     if (!called)
       {
-        _dbus_verbose("TODO: memory allocation testing errors disabled for now\n");
+        _dbus_verbose_raw ("TODO: memory allocation testing errors disabled for now\n");
         called = 1;
       }
     return FALSE;
